* LISTRU.PRG - Vers. 1.0d
* W.J. O'Connor
*
* This program creates and prints out a report of the structure of a table.  It will create a
* cursor that holds the fields in a table and a report is done on the cursor.
*
*** Parameters:
* cTable 		- The name of the table to be reported on.
* cReportType 	- Selects one of Two Parameters "P" for Printing the Report and "V" for Previewing
*				  the Report
* lPrintDetail	- Determines whether LISTRU will print the table and field validations.
*** Changes
* 951011		- Fixed the help facility.
* 951011		- Improved the table handling.  No Longer leaves the cursor open and VFP in the wrong work area
* 951011		- Mildly reformatted the reports.
* 951013		- If no parameters are passed and table is not open in current area, will ask for table to view
*				- and ask if to list table and field validations.
* 951017		- The cTable parameter no longer needs to be passed.  Expanded the help with examples.
* 951115		- Modified to add index tags and expressions (M. Huebner)

LPARAMETER cTable, cReportType, lPrintDetail
LOCAL aTemp[1], lnFieldCount, lTableUsed, cAlias, lPrint, cOldExcl, lTags, j

lTableUsed = .F.
IF PCOUNT() = 0			&& No Parameters passed.
  IF NOT EMPTY(DBF())
    cTable = DBF()
  ELSE
    cTable = GETFILE('DBF', 'Select Table to List')
    IF ATC("UNTITLED",cTable) > 0 OR EMPTY(cTable)
      RETURN
    ENDIF
    lPrint = MESSAGEBOX("Print the Table Structure?",256+32+4) = 6
    IF lPrint
      cReportType = "P"
    ELSE
      cReportType = "V"
    ENDIF
    lPrintDetail = MESSAGEBOX("List Table and Field Validations?",256+32+4) = 6
  ENDIF
ENDIF

IF TYPE("cTable") = "C"
  IF cTable <> "?"
    IF NOT USED(cTable)
      IF "\" $ cTable OR "." $ cTable OR ":" $ cTable
        cAlias = STRIPPATH(STRIPEXT(cTable))
      ELSE
        cAlias = cTable
      ENDIF
      USE (cTable) SHARED NOUPDATE IN 0 ALIAS (cAlias)
      lTableUsed = .T.
    ENDIF
  ENDIF
ELSE
  IF NOT EMPTY(Alias())
    cTable = ALIAS()
    cAlias = ALIAS()
  ELSE
    = MESSAGEBOX("If No Table Name is passed, " + CHR(13) + ;
      "There must be a Table open in the current Work Area!",16,"Error!")
    RETURN
  ENDIF
ENDIF

IF TYPE("cReportType") # "C" AND cTable <> "?"
  cReportType = "V"		&& Preview Mode
ENDIF

IF cTable = "?"
  DO LISTRUHELP
  RETURN
ENDIF

* If the cursor exists, close before recreating the cursor.
IF USED("LISTRU")
  USE IN LISTRU
ENDIF
* Create the cursor.
SELECT 0
CREATE CURSOR LISTRU ;
    (FieldName C (32), ;
     FieldType C (1), ;
     FieldLen  N (3), ;
     FieldDec  N (3), ;
     NullAllow L, ;
     Translate L, ;
     FieldVal  M, ;
     FieldText M, ;
     FieldDef  M, ;
     TableVal  M, ;
     TableText M)
     
CREATE CURSOR LISTRU1;
	(FieldTags M)

SELECT (cAlias)
lnFieldCount = AFIELDS(aTemp)
lTags = ""
FOR j = 1 TO 999
	IF EMPTY(TAG(j))
		EXIT
	ELSE
		lTags = lTags + TAG(j) + "	=	" + KEY(j) + CHR(13) + CHR(10)
	ENDIF
ENDFOR
SELECT LISTRU1
INSERT INTO LISTRU1 (FieldTags) VALUES (lTags)
select LISTRU
IF lnFieldCount > 0
  INSERT INTO LISTRU FROM ARRAY aTemp
  DO CASE
    CASE cReportType = "P"			&& Print the Report
      IF lPrintDetail
        REPORT FORM LISTRU1 TO PRINTER NOCONSOLE
      ELSE
        REPORT FORM LISTRU2 TO PRINTER NOCONSOLE
      ENDIF
    OTHERWISE						&& PreView the Report
      IF lPrintDetail
        REPORT FORM LISTRU1 PREVIEW NOCONSOLE
      ELSE
        REPORT FORM LISTRU2 PREVIEW NOCONSOLE
      ENDIF
  ENDCASE
ENDIF
USE IN LISTRU
USE IN LISTRU1
IF lTableUsed
  USE IN (cAlias)
ELSE
  SELECT (cTable)
ENDIF
RETURN

PROCEDURE LISTRUHELP
CLEAR
SET TEXTMERGE ON
\ LISTRU.APP
\ Version 1.0d
\ Copyright (c) 1995 ContrAcct Systems Corp.
\ William J. O'Connor
\
\ This program is distributed as FreeWare.
\ The author makes no warranties expressed or implied
\ as to the suitability of use of this program.  Use
\ at your own risk.  This program may be distributed
\ freely, so long as no modifications are made to the
\ application or associated files.  Thank you for your
\ support.
\ Suggestions for changes and refinements should be
\ sent via Compuserve E-Mail to 75231,3643 or posted
\ to Compuserve's VFOX forum.
\
WAIT
\ LISTRU creates a report on the structure of a Visual
\ FoxPro 3.0 table.
\   Currently, using LIST STRUCTURE creates a listing
\ that is poorly formatted and hard to read.  LISTRU
\ creates a formatted report that is much easier to read.
\ 
\ LISTRU accepts three parameters:
\
\   Parameter 1 is the Name of the Table.  This will default
\ to the currently opened table if it is not passed.  This 
\ must be a character expression if passed.  The table will be
\ USEd if the table isn't already open.  If the table name is
\ not passed, the table in the current work area will be used.
\ If LISTRU must open the table to report on it, the table will
\ be closed when LISTRU ends.
\
\ Parameter 2 is either a "P" or a "V":
\ "P" Sends the report to the printer.  "V" Previews the Report.
\ This will default to "V" if it is not passed.
\
\ Parameter 3 is a logical value that if .T. shows the Field and
\ Table Validations.  If .F., the Field and Table Validations
\ are skipped.  Defaults to .F.
\
\ If no parameters are passed, LISTRU will use the table opened in
\ the current work area, using all other defaults.  If no table is
\ open in the current area, you will be prompted to select a table,
\ whether you want to view the Field and Table Validations, and if
\ you want the structure printed.
\
WAIT
\ Examples:
\ = LISTRU("MYTABLE")
\   This will select or use the table named "MYTABLE", and show it in
\   Preview mode with no Table and Field Validations.
\
\ = LISTRU("MYTABLE","P")
\   This will select or use the table named "MYTABLE", and print it to
\   the printer.
\
\ = LISTRU("MYTABLE","V",.T.)
\   This will select or use the table named "MYTABLE" and Preview it
\   including the Table and Field Validations.
\
\ = LISTRU(,"V")
\   This will take the table in the currently opened work area and
\   Preview it without Table and Field Validations.
\
SET TEXTMERGE OFF
RETURN
ENDPROC

FUNCTION STRIPEXT
* Strips the extension from the passed file name
LPARAMETERS pcFileName
LOCAL lcFileName

IF PCOUNT() < 1
  WAIT WINDOW TIMEOUT 2 "File Name Parameter Not Passed: StripExt"
ENDIF

IF "." $ pcFileName
  lcFileName = SUBSTR(pcFileName, 1,RAT(".", pcFileName) - 1)
ELSE
  lcFIleName = pcFileName
ENDIF
RETURN lcFileName
ENDFUNC

FUNCTION STRIPPATH
* Strips a path from a passed file name
LPARAMETERS pcFileName
LOCAL lcFileName
IF PCOUNT() < 1
  WAIT WINDOW TIMEOUT 2 "File Name Parameter Not Passed: StripPath"
ENDIF
DO CASE
  CASE "\" $ pcFileName				&& Path Contained in FileName
    lcFileName = SUBSTR(pcFileName, RAT("\", pcFileName) + 1)
  CASE ":" $ pcFileName				&& Drive Spec Contained in FileName
    lcFileName = SUBSTR(pcFileName, RAT(":", pcFileName) + 1)
  OTHERWISE							&& No Drive or Path Contained in FileName
    lcFileName = pcFileName
ENDCASE

RETURN lcFileName
ENDFUNC
